@@ -1,8 +1,7 @@
 # @title DEEPHYS
 from cbor2 import dump
 from dataclasses import dataclass, asdict
-from typing import List, Optional
-from typing import List, Optional
+from typing import List, Optional, Mapping
 import numpy
 import torch
 import struct
@@ -65,11 +64,56 @@ class Model(DEEPHYSData):
 
 
 def import_torch_dataset(name, dataset, classes, state, model):
-    imageList = []
+    """
+    Conveniently calls import_test_data with PyTorch data.
+
+    :param name: the name of the dataset
+    :type name: str
+    :param dataset: contains pixel data of images
+    :type dataset: torch.utils.data.DataLoader
+    :param classes: an ordered list of strings representing class names
+    :type classes: list
+    :param state: a 3D array of floats [layers,neurons,activations]
+    :type state: list
+    :param model: the model structure
+    :type model: deephys.deephys.Model
+    :return: a formatted data object which may be saved to a file
+    :rtype: deephys.deephys.Test
+    """
+    pixelDataList = []
+    groundTruthList = []
     for i in range(len(dataset)):
         image, target = dataset[i]
         if torch.is_tensor(target):
             target = target.item()
+        pixelDataList.append(image)
+        groundTruthList.append(target)
+    return import_test_data(name, pixelDataList, groundTruthList, classes, state, model)
+
+
+def import_test_data(name, classes, state, model, pixel_data, ground_truths):
+    """
+    Prepare test results for Deephys
+
+    :param name: the name of the dataset
+    :type name: str
+    :param classes: an ordered list of strings representing class names
+    :type classes: list
+    :param state: a 3D array of floats [layers,neurons,activations]
+    :type state: list
+    :param model: the model structure
+    :type model: deephys.deephys.Model
+    :param pixel_data: an ordered list of image pixel data [images,channels,dim1,dim2]
+    :type pixel_data: List[List[List[List]]]
+    :param ground_truths: an ordered list of ground truths
+    :type ground_truths: List[int]
+    :return: a formatted data object which may be saved to a file
+    :rtype: deephys.deephys.Test
+    """
+    imageList = []
+    for i in range(len(pixel_data)):
+        image = pixel_data[i]
+        target = ground_truths[i]
         image = image * 255
         mn = torch.min(image)
         mx = torch.max(image)
@@ -102,6 +146,7 @@ def import_torch_dataset(name, dataset, classes, state, model):
                         )
                     )
                 ),
+                features=None,
             )
         )
     test = Test(name=name, suffix=None, images=imageList)
@@ -114,6 +159,7 @@ class ImageFile:
     categoryID: int
     category: str
     data: List[bytearray]  # R8G8B8
+    features: Optional[Mapping[str, str]]
     activations: Model.ModelState
 
 
@@ -140,11 +186,11 @@ class Stopwatch:
         print(f"{self.name} took {self.duration_secs} seconds")
 
     def stop_and_report(self):
-        self.stop()
-        self.report()
+        self.stop(null)
+        self.report(null)
 
 
 def start_stopwatch(name):
     sw = Stopwatch(name)
     sw.start()
-    return sw
\ No newline at end of file
+    return sw
